#### Library import ####
library(car)
library(nnet)
library(plyr)
library(fields)
library(beepr)
library(makedummies)
library(ggplot2)

#### Function ####
cross_entropy_f <- function (x,r){
  pre_pro_tmp <- predict(r,new=x,type="pro")
  ans_all <- x[,1]
  tmp <- data.frame(ans_all, pre_pro_tmp)
  dmy <- makedummies(tmp, basal_level = TRUE,col = "ans_all")
  log_pre_pro_tmp <- log(pre_pro_tmp + 10^-7)
  tmp <- -sum(dmy*log_pre_pro_tmp)
  return(tmp)
} 


#### Data import ####
df <- read.csv("MnzData.csv") 
#### Pre-processing ####
x <- df[,-c(18,19,20)]
dataNum <- nrow(x) # Number of Dataset
x[,-1] <- scale(x[,-1])# Normalization
# Box-Cox transfromation
for (i in 1:15) {
lambda <- powerTransform(x[,i+1]-min(x[,i+1]) + 0.01)
x[,i+1] <- bcPower(x[,i+1]-min(x[,i+1]) + 0.1, lambda$roundlam)
} 
####  Multinomial logistic regression ####
for (i in 16:1) {
  if (i == 16){
    txt <- paste("N=",i," START: ",format(Sys.time(), "%Y/%m/%d %H:%M:%OS"))
    write(txt, file="progress.txt")
    Element_Number <- data.frame(UsedNumber=i)
    r <- multinom(formula=label ~ ., data=x)  # MLR
    AIC <- AIC(r)  #AIC
    coe <- coefficients(r) #regression coefficients
    w_Grtpre <- as.data.frame(t(coe[1,])) # coefficients for Grt-pre metamorphic
    w_Ign <- as.data.frame(t(coe[2,]))  # coefficients for igneous
    w_Leu <- as.data.frame(t(coe[3,]))  # coefficients for leucogranite
    pre <- predict(r,data=x) # Prediction
    Conf_mat <- table(x[,1],pre) # Confusion Matrix
    DR_tmp <- (Conf_mat[1,1]+Conf_mat[2,2]+Conf_mat[3,3]+Conf_mat[4,4])/dataNum # calucalate Discrimination rate
    DR <- DR_tmp
    # Cross validation
    for (k in 1:dataNum) {
      test <- x[k,]
      training <-  x[-k,]
      r_tmp <- multinom(label ~ ., data=training)
      AIC_CV_tmp <- r_tmp$AIC
      ans <- x[k,1]
      pre_class_tmp <- predict(r_tmp,new=test,type="class")
      Check_tmp <- match(ans, pre_class_tmp)
      pre_pro_tmp <- predict(r_tmp,new=test,type="pro")
      cross_entropy_single_tmp <- -log(pre_pro_tmp[ans])
      cross_entropy_all_tmp <- cross_entropy_f(x,r_tmp)
      if (k ==1){
        Check <- Check_tmp
        AIC_CV <- AIC_CV_tmp
        cross_entropy_single <- cross_entropy_single_tmp
        cross_entropy_all <- cross_entropy_all_tmp
      } else {
        Check <- rbind(Check, Check_tmp)
        AIC_CV <- AIC_CV + AIC_CV_tmp
        cross_entropy_single <- cross_entropy_single + cross_entropy_single_tmp
        cross_entropy_all <- cross_entropy_all + cross_entropy_all_tmp
      }
    }
    summary_DR_CV <- length(Check[!is.na(Check)])/length(Check)
    summary_AIC_CV <- AIC_CV/dataNum
    summary_cross_entropy_single <- cross_entropy_single/dataNum
    summary_cross_entropy_all <- cross_entropy_all/dataNum
  } else { 
    txt <- rbind(txt,paste("N=",i," START: ",format(Sys.time(), "%Y/%m/%d %H:%M:%OS")))
    write(txt, file="progress.txt")
    beep(sound = 3, expr = NULL)
    Sys.sleep(3)
    columnNum <- c(2:17)
    combination <- combn(columnNum,(16-i)) # Choose elements to be removed
    total_combination <-  ncol(combination) # Number of comibination
    for (k in 1:total_combination) {
      Element_Number <- rbind(Element_Number,i)
      Remcolm <- (combination[,k])
      xn <- x[, -Remcolm]
      r <- multinom(formula=label ~ ., data=xn)
      AIC <- rbind(AIC,r$AIC)  #AIC
      coe <- coefficients(r)#regression coefficients
      w_Grtpre <- rbind.fill(w_Grtpre, as.data.frame(t(coe[1,]))) # coefficients for Grt-pre metamorphic
      w_Ign <- rbind.fill(w_Ign, as.data.frame(t(coe[2,])) ) # coefficients for igneous
      w_Leu <- rbind.fill(w_Leu, as.data.frame(t(coe[3,])) ) # coefficients for leucogranite
      pre <- predict(r,data=x) # Prediction
      Conf_mat <- table(x[,1],pre) # Confusion Matrix
      DR_tmp <- (Conf_mat[1,1]+Conf_mat[2,2]+Conf_mat[3,3]+Conf_mat[4,4])/dataNum # calucalate Discrimination rate
      DR <- rbind(DR,DR_tmp)
      #cross validation
      for (l in 1:dataNum) {
        test <- xn[l,]
        training <-  xn[-l,]
        r_tmp <- multinom(label ~ ., data=training)
        AIC_CV_tmp <- r_tmp$AIC
        ans <- xn[l,1]
        pre_class_tmp <- predict(r_tmp,new=test,type="class")
        Check_tmp <- match(ans, pre_class_tmp)
        pre_pro_tmp <- predict(r_tmp,new=test,type="pro")
        cross_entropy_single_tmp <- -log(pre_pro_tmp[ans])
        cross_entropy_all_tmp <- cross_entropy_f(x,r_tmp)
        if (l ==1){
          Check <- Check_tmp
          AIC_CV <- AIC_CV_tmp
          cross_entropy_single <- cross_entropy_single_tmp
          cross_entropy_all <- cross_entropy_all_tmp
        } else {
          Check <- rbind(Check, Check_tmp)
          AIC_CV <- AIC_CV + AIC_CV_tmp
          cross_entropy_single <- cross_entropy_single + cross_entropy_single_tmp
          cross_entropy_all <- cross_entropy_all + cross_entropy_all_tmp
        }
      }
      DR_CV <- length(Check[!is.na(Check)])/length(Check)
      AIC_CV <- AIC_CV/dataNum
      cross_entropy_single <- cross_entropy_single/dataNum
      cross_entropy_all <- cross_entropy_all/dataNum
      summary_DR_CV <- rbind(summary_DR_CV, DR_CV)
      summary_AIC_CV <- rbind(summary_AIC_CV,AIC_CV)
      summary_cross_entropy_single <- rbind(summary_cross_entropy_single, cross_entropy_single)
      summary_cross_entropy_all <- rbind(summary_cross_entropy_all, cross_entropy_all)
      if (k %% 500 == 0) {
        txt <- rbind(txt,paste("N=",i,"Done: ",k/length(combination)*100, "% ", format(Sys.time(), "%Y/%m/%d %H:%M:%OS")))
        write(txt, file="progress.txt")
      }
    }
  }
  results <- cbind(Element_Number,DR,AIC,
                   summary_DR_CV,summary_AIC_CV,
                   summary_cross_entropy_single,summary_cross_entropy_all,
                   w_Grtpre,w_Ign,w_Leu)
  colnames(results) <- c("Number of Elements","DR","AIC",
                         "DR_CV","AIC_CV","CE_single","CE_all",
                         "GP_intercept","GP_La","GP_Ce","GP_Pr","GP_Nd","GP_Sm","GP_Eu","GP_Gd","GP_Tb","GP_Dy","GP_Ho","GP_Er","GP_Tm","GP_Yb","GP_Lu","GP_Th","GP_U",
                         "Ig_intercept", "Ig_La","Ig_Ce","Ig_Pr","Ig_Nd","Ig_Sm","Ig_Eu","Ig_Gd","Ig_Tb","Ig_Dy","Ig_Ho","Ig_Er","Ig_Tm","Ig_Yb","Ig_Lu","Ig_Th","Ig_U",
                         "Leu_intercept", "Leu_La","Leu_Ce","Leu_Pr","Leu_Nd","Leu_Sm","Leu_Eu","Leu_Gd","Leu_Tb","Leu_Dy","Leu_Ho","Leu_Er","Leu_Tm","Leu_Yb","Leu_Lu","Leu_Th","Leu_U")  
  rownames(results) <- c(1:nrow(results))
  order_rank <- order(results$DR_CV, -1*results$AIC_CV, decreasing = T)
  results_sort <- results[order_rank,]
  write.csv(results_sort,"MLR.csv", fileEncoding = "CP932")
}      
  
#### DATA EXPORT ####
results <- cbind(Element_Number,DR,AIC,
                 summary_DR_CV,summary_AIC_CV,
                 summary_cross_entropy_single,summary_cross_entropy_all,
                 w_Grtpre,w_Ign,w_Leu)
colnames(results) <- c("Number of Elements","DR","AIC",
                       "DR_CV","AIC_CV","CE_single","CE_all",
                       "GP_intercept","GP_La","GP_Ce","GP_Pr","GP_Nd","GP_Sm","GP_Eu","GP_Gd","GP_Tb","GP_Dy","GP_Ho","GP_Er","GP_Tm","GP_Yb","GP_Lu","GP_Th","GP_U",
                       "Ig_intercept", "Ig_La","Ig_Ce","Ig_Pr","Ig_Nd","Ig_Sm","Ig_Eu","Ig_Gd","Ig_Tb","Ig_Dy","Ig_Ho","Ig_Er","Ig_Tm","Ig_Yb","Ig_Lu","Ig_Th","Ig_U",
                       "Leu_intercept", "Leu_La","Leu_Ce","Leu_Pr","Leu_Nd","Leu_Sm","Leu_Eu","Leu_Gd","Leu_Tb","Leu_Dy","Leu_Ho","Leu_Er","Leu_Tm","Leu_Yb","Leu_Lu","Leu_Th","Leu_U")  
rownames(results) <- c(1:nrow(results))
order_rank <- order(results$DR_CV, -1*results$AIC_CV, decreasing = T)
results_sort <- results[order_rank,]
write.csv(results_sort,"MLR.csv", fileEncoding = "CP932")


#################     Visualliization       ################

#########  Histogram of Frequency  ##########
p_1 <- ggplot(results_sort,aes(x=DR_CV)) +
       theme_bw() +
       geom_histogram(fill="hotpink",colour="black",binwidth = 0.01) +
       xlim(0.5, 1) +
        scale_y_continuous(expand = c(0,0), limits = c(0,15000))
plot(p_1)
dev.off()
ggsave(file = "histogram.eps", plot = p_1)

breaks <- seq(0.4,1,0.01)
h <- hist(result_sort$DR_CV,breaks = breaks)
n <- length(h$counts) # 階級の数
class_names <- NULL # 階級の名前格納用
for(i in 1:n) {
  class_names[i] <- paste(h$breaks[i], "～", h$breaks[i+1])
}
table <- data.frame(class=class_names, frequency=h$counts, density=h$density)
write.csv(table,file ="Histogram_freq_density.csv")


########## DR vs. Numebr of used elements ##########
tmp <- results_sort
tmp$Number.of.Elements <- as.factor(tmp$Number.of.Elements)
p_2 <- ggplot(tmp,aes(x=Number.of.Elements, y=DR_CV)) +
  theme_bw() +
  geom_boxplot() 
p_3 <- ggplot(tmp,aes(x=Number.of.Elements, y=CE_all)) +
  theme_bw() +
  geom_boxplot() 
p_4 <- ggplot(tmp,aes(x=Number.of.Elements, y=AIC_CV)) +
  theme_bw() +
  geom_boxplot() 
p_5 <- grid.arrange(p_2, p_3, p_4, nrow=1)
ggsave(file = "boxplot.eps", plot = p_5)


##########  Heat map of regression coefficients  ##########
postscript("heatmap.eps")
par(mfrow = c(3, 1))
rwb <- colorRampPalette(colors = c("blue", "white", "red"))
mat1 <-t(results_sort[,7:22])
rownames(mat1) <- c("La","Ce","Pr","Nd","Sm","Eu","Gd","Tb","Dy","Ho","Er","Tm","Yb","Lu","Th","U")
colnames(mat1) <- (1:nrow(results_sort))
mat1 <- mat1[,1:100]

# Lasagna plot
lasagna.leg <- function(X, col=palette, axes=F, ...){
  fields::image.plot(t(X)[,(nrow(X):1)], col=col, axes=axes, ... )
  box()
}
lasagna.axis <- function(mat1){
  axis(side=1,at=seq(0,1,length=ncol(mat1)),labels=colnames(mat1))
  axis(side=2,at=seq(1,0,length=nrow(mat1)),labels=rownames(mat1),las=TRUE,cex.axis=0.8)
}

colorbar <- seq(-100,100,20)
lasagna.leg(mat1, col=rwb(10),axes = FALSE, main="Grt-present / w", breaks=colorbar)
#lasagna.lab(mat1)
lasagna.axis(mat1)

mat1 <-t(results_sort[,24:39])
rownames(mat1) <- c("La","Ce","Pr","Nd","Sm","Eu","Gd","Tb","Dy","Ho","Er","Tm","Yb","Lu","Th","U")
colnames(mat1) <- (1:nrow(results_sort))
mat1 <- mat1[,1:100]
lasagna.leg(mat1, col=rwb(10),axes = FALSE, main="Igneous / w",breaks=colorbar)
lasagna.axis(mat1)

mat1 <-t(results_sort[,41:56])
rownames(mat1) <- c("La","Ce","Pr","Nd","Sm","Eu","Gd","Tb","Dy","Ho","Er","Tm","Yb","Lu","Th","U")
colnames(mat1) <- (1:nrow(results_sort))
mat1 <- mat1[,1:100]
lasagna.leg(mat1, col=rwb(10),axes = FALSE, main="Leucogranite / w",breaks=colorbar)
lasagna.axis(mat1)
dev.off()

